#include "Grid.H"

#include "Logger.H"

extern Logger logger;

Grid::Grid(unsigned int wdth, unsigned int hght):
	_width(wdth),
	_height(hght) {
	_grd=new Char[_height * _width];             
}

Grid::Grid(const Grid &g):
	_width(g._width), 
	_height(g._height) {
	_grd=new Char[_height * _width];             
	for (unsigned int row=0; row<_height; row++)
		for (unsigned int col=0; col<_width; col++)
			_grd[(row*_width)+col]= g._grd[(row*_width)+col];
}

Grid::~Grid() {
	delete[] _grd;
}

void Grid::fill(Char chr) {
	for (unsigned int row=0; row<_height; row++)
		for (unsigned int col=0; col<_width; col++)
			_grd[(row*_width)+col]=chr;
}

void Grid::fill(unsigned int col, Char chr) {
	if (col >= _width)
		return;
	for (unsigned int row=0; row<_height; row++)
		_grd[(row*_width)+col]=chr;
}

void Grid::fill(Char chr, unsigned int row) {
	if (row >= _height)
		return;
	for (unsigned int col=0; col<_width; col++)
		_grd[(row*_width)+col]=chr;
}

void Grid::fill(Char chr, unsigned int col1, unsigned int row1,
		unsigned int col2, unsigned int row2) {
	if (col1>col2) {
		col1+=col2;
		col2=col1-col2;
		col1-=col2;
	}
	if (row1>row2) {
		row1+=row2;
		row2=row1-row2;
		row1-=row2;
	}
	if ((col1>=_width) || (row1>=_height))
		return;
	if (col2>=_width)
		col2=_width-1;
	if (row2>=_height)
		row2=_height-1;
	for (unsigned int x=col1; x<=col2; x++) {
		for (unsigned int y=row1; y<=row2; y++) {
			_grd[(y*_width)+x]=chr;
		}
	}
}

int Grid::set(unsigned int col, unsigned int row, Char chr) {
	if ((col >= _width) || (row >= _height))
		return (-1);
	_grd[(row*_width)+col]=chr;
	return 0;
}

int Grid::set(int col, int row, const Grid &grd) {
	unsigned int
		i, j, k, m, xonscr, yonscr;

	xonscr=0;
 	yonscr=0;
	if ((col >= (signed)_width) || (row >= (signed)_height) ||
		((col+(signed)grd.width()) < 0) || ((row+(signed)grd.height()) < 0))
		return (-1);

	while (col < 0) {
		col++;
		xonscr++;
	}
	while (row < 0) {
		row++;
		yonscr++;
	}

	for (k=row,m=yonscr; (k<_height) && (m<grd.height()); k++,m++) {
		for (i=col,j=xonscr; (i<_width) && (j<grd.width()); i++,j++) {
			if (grd.get(j,m).isTransparent())
				continue;
			_grd[(k*_width)+i]=grd.get(j,m);
		}
	}
	return 0;
}

Char Grid::get(unsigned int col, unsigned int row) const {
	if ((col >= _width) || (row >= _height))
		return (_nul);
	return (_grd[(row*_width)+col]);
}
